@@ -5,7 +5,9 @@
 
 import datetime
 import itertools
-from typing import Dict, List, Sequence, Tuple, Union
+import warnings
+from collections import Counter
+from typing import Any, Dict, List, Optional, Sequence, Tuple, Union
 
 import numpy as np
 import pandas as pd
@@ -195,6 +197,135 @@ def multi_objective_visualization(
     return fig
 
 
+def parallel_coordinate_plot_parameters(
+    evaluations: List[Evaluation],
+    columns: Optional[List[str]] = None,
+    color_by: Optional[str] = None,
+):
+    """Create an interactive parallel coordinate plot.
+
+    Useful to investigate relationships in a higher dimensional search space and the
+        optimization's objective(s).
+
+    Args:
+        evaluations: Evaluations to plot.
+        columns: Names of columns to show. Can contain parameter names, objective names
+            and settings keys. If `None`, all parameters, objectives and settings are
+            displayed.
+        color_by: Parameter name, objective name or settings key. The corresponding
+            column will be shown at the very right, it's value will be used for the
+            color scale. If `None`, all lines have the same color.
+
+    Returns:
+        Plotly figure
+
+    Raised:
+        NoSuccessfulEvaluationsError: In case `evaluations` does not contain at least
+            one successful evaluation (an evaluation with objective value != `None`).
+    """
+    if not evaluations:
+        raise NoSuccessfulEvaluationsError
+
+    # Prepare dataframe for visualization
+    df = evaluations_to_df(evaluations)
+
+    # Drop unused columns and indices
+    if "settings" in df.columns:
+        df = df[["configuration", "settings", "objectives"]]
+        settings_cols = df["settings"].columns.to_list()
+    else:
+        df = df[["configuration", "objectives"]]
+        settings_cols = []
+    objective_cols = df["objectives"].columns.to_list()
+    df = df.droplevel(0, axis=1)
+
+    # If no columns are specified, use all:
+    if not columns:
+        columns = df.columns.to_list()
+
+    if color_by and color_by not in columns:
+        raise ValueError(
+            f"Unknown column name in color_by='{color_by}'. Please make sure, that this"
+            + "column name is correct and one of the visible columns."
+        )
+
+    ambigious_columns = [k for k, v in Counter(df[columns].columns).items() if v > 1]
+    if ambigious_columns:
+        raise ValueError(
+            "All columns to plot must have a unique name, but those are ambigious: "
+            + f"{ambigious_columns}. Either rename parameters/settings/objective to "
+            + "be unique or provide only the unambigious ones as `columns` argument."
+        )
+
+    # Prepare a coordinate (vertical line) for every column
+    coordinates = []
+    colored_coordinate = {}
+    for column in columns:
+        coordinate: Dict[str, Any] = {}
+
+        if column in objective_cols:
+            coordinate["label"] = f"<b>Objective: {column}</b>"
+        elif column in settings_cols:
+            coordinate["label"] = f"Setting: {column}"
+        else:
+            coordinate["label"] = column
+
+        parameter_type = df[column].dtype.name
+        if parameter_type.startswith("float") or parameter_type.startswith("int"):
+            # Handling floats and integers the same, because unfortunately it's hard to
+            # use integers only for ticks and still be robust regarding a large range
+            # of values.
+            coordinate["values"] = df[column]
+        elif parameter_type in ["object", "bool"]:
+            # Encode categorical values to integers. Unfortunately, ordinal parameters
+            # loose there ordering, as there is no information about the order in the
+            # evaluations.
+            df[column] = df[column].astype("category")
+            categories = df[column].cat.categories.to_list()
+            encoded_categories = list(range(len(categories)))
+            df[column].cat.categories = encoded_categories
+            # Use integer encodings for scale and category values as tick labels
+            coordinate["ticktext"] = categories
+            coordinate["tickvals"] = encoded_categories
+            coordinate["values"] = df[column].astype("str")
+        else:
+            warnings.warn(
+                f"Ignoring column with unknown type: {column}<{parameter_type}>"
+            )
+            continue
+
+        if column == color_by:
+            colored_coordinate = coordinate
+        else:
+            coordinates.append(coordinate)
+
+    # Append colored coordinate to the end (right)
+    if colored_coordinate:
+        coordinates.append(colored_coordinate)
+
+    # Plot
+    return go.Figure(
+        data=go.Parcoords(
+            line=dict(
+                # Color lines by objective value
+                color=df[color_by] if color_by else None,
+                colorscale=px.colors.diverging.Tealrose,
+                showscale=True,
+                # Use colorbar as kind of colored extension to the axis
+                colorbar=dict(
+                    thickness=16,
+                    x=1,
+                    xpad=0,
+                    ypad=1,
+                    showticklabels=False,
+                ),
+            ),
+            dimensions=coordinates,
+        ),
+        layout=dict(title="[BBO] Parallel coordinates plot"),
+    )
+
+
 class Visualizer:
     def __init__(self, evaluations: List[Evaluation], objective: Objective):
         times_finished = np.array(utils.get_times_finished(evaluations))
